#ifndef PARTICLE_HPP
#define PARTICLE_HPP

#include <iostream>
using std::ostream;

#include <cmath>
using std::sqrt;

#include "vec2.hpp"

class particle
{
public:
	// member data
	double mass;
	vec2 r; // position
	vec2 v; // velocity
	vec2 a; // acceleration
	double t; // current time;

	// constructor
	particle(double m_, vec2 r_, vec2 v_) : mass(m_), r(r_), v(v_), a(0,0), t(0) {}
	
	// methods (or member functions)
	void evolve(void (&method)(particle&,double), double dt)
	{
		// *this refers to the active particle object
		method(*this, dt);
	}

	void draw(void)
	{
		if (mass < 1.5)
		{
			glPointSize(2.0*mass); 
			glBegin(GL_POINTS); 
				glColor3f(1.0, 1.0, 1.0); 
				glVertex2f(r.x,r.y);
			glEnd();
		}
		else
		{		
			const float radius = 0.005f*sqrt(mass);
			glBegin(GL_LINE_LOOP); 
				glColor3f(1.0, 0.5, 0.25); 
				for(int j = 0; j < 360; ++j)
					glVertex2f(r.x+radius*cos(j*M_PI/180.0f),r.y+radius*sin(j*M_PI/180.0f));
			glEnd();
		}
		glFlush();
	}
	
	// a friend function does not belong to the class but is affiliated with it
	friend ostream& operator<<(ostream& os, const particle &p)
	{
		os.precision(8);
		os << setw(20) << p.t
		    << setw(20) << p.r
		    << setw(20) << p.v
		    << setw(20) << p.a;
		return os;
	}
};

#endif // PARTICLE_HPP
